/*
 *  ScanLine
 *
 *  Copyright (C) 2016 Omega2058
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <pspkernel.h>
#include <pspsysmem_kernel.h>
#include <psputilsforkernel.h>
#include <string.h>
#include "sctrl.h"

PSP_MODULE_INFO("ScanLine", PSP_MODULE_KERNEL, 0, 1);

#define LINESIZE 512				//in long
#define FRAMESIZE 0xAA000		//in byte
#define GREEN	0x0000FF00L

typedef unsigned int UINT;

typedef struct {
	UINT left;
	UINT right;
	UINT top;
	UINT bottom;
} RECT;

RECT pixel = { 0 };
unsigned char *vramtop=(unsigned char *)0x04000000;
unsigned long drawframe;
static STMOD_HANDLER previous = NULL;
u32 nextmod = 0;

// get vram address for character position
unsigned char *GetVramAddr(unsigned long x,unsigned long y)
{
	return vramtop+(drawframe?FRAMESIZE:0)+x*4+y*LINESIZE*4+0x40000000;
}

void FillRect(RECT *xirect, unsigned long xicolour)
{
	unsigned long *vptrl;
	unsigned char *vptrc;
	int y;

	for (y = xirect->top; y < xirect->bottom; y++)
	{
		int x = xirect->left;
		vptrc = GetVramAddr(x,y);
		vptrl = (unsigned long*)vptrc;

		while (x < xirect->right)
		{
			*vptrl++ = xicolour;
			x++;
		}
	}
}

void drawPixel(int x, int y, int sizex, int sizey, unsigned long color) {
	pixel.top = y;
	pixel.left = x;
	pixel.bottom = y+sizey;
	pixel.right = x+sizex;

	FillRect(&pixel, color);
}

int module_start_handler(SceModule2 * module)
{
	int ret = previous ? previous(module) : 0;

	//signal that the main game module has loaded.
	if (nextmod == 1)
		nextmod++;

	//signal that sceKernelLibrary has loaded, the next module should be the main game module.
	if (!strcmp(module->modname, "sceKernelLibrary"))
		nextmod++;

	return ret;
}

int thread_start(SceSize args, void *argp)
{
	int x = 0;
	int y = 0;
	unsigned long *vptrl;
	unsigned char *vptrc;
	unsigned long xicolour;
	previous = sctrlHENSetStartModuleHandler(module_start_handler);

	//wait until the main game module starts
	while (nextmod != 2)
		sceKernelDelayThread(1000);

	while(1) {
		sceKernelDelayThread(1500);


		for(y = 0; y < 272; y+= 4) {
			for(x = 0; x < 480; x++) {
				// Get the pixel of the next line
				vptrc = GetVramAddr(x,y+1);
				vptrl = (unsigned long*)vptrc;
				xicolour = *vptrl;
				// Copies and place it over the current one.
				// Basically it's a simple C&P, but maybe someone else who knows more can maybe do it better?
				drawPixel(x, y, 1, 1, xicolour);
			}
		}
	}

	return 0;
}

int module_start(SceSize args, void *argp)
{
	SceUID thid = sceKernelCreateThread("ScanLine_Main", thread_start, 0x22, 0x2000, 0, NULL);

	if (thid >= 0)
		sceKernelStartThread(thid, args, argp);

	return 0;
}

int module_stop(SceSize args, void *argp)
{
	return 0;
}
